/*
Copyright (c) 2003 Bruno T. C. de Oliveira

LICENSE INFORMATION:
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
Copyright (c) 2002 Bruno T. C. de Oliveira

INFORMAES DE LICENA:
Este programa  um software de livre distribuio; voc pode
redistribu-lo e/ou modific-lo sob os termos da GNU General
Public License, conforme publicado pela Free Software Foundation,
pela verso 2 da licena ou qualquer verso posterior.

Este programa  distribudo na esperana de que ele ser til
aos seus usurios, porm, SEM QUAISQUER GARANTIAS; sem sequer
a garantia implcita de COMERCIABILIDADE ou DE ADEQUAO A
QUALQUER FINALIDADE ESPECFICA. Consulte a GNU General Public
License para obter mais detalhes (uma cpia acompanha este
programa, armazenada no arquivo COPYING).
*/

#include <bores.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <zlib.h>

#include "document.h"

#define MYSYNTAX \
   "Syntax: ae2aes [-o <output_file>] inputfile\n" \
   "\n" \
   "   Converts an aewan file to a simple text-based format called AES\n" \
   "\n" \
   "   -o : writes output to specified file rather than stdout\n" \
   "   -h : prints this help text\n" 

void rtfm() {
   fprintf(stderr, MYSYNTAX);
   exit(1);
}

int main(int argc, char **argv) {
   int i, x, y, ch;
   static char optstring[] = "o:h";
   char *outputfile = NULL;
   char *inputfile;
   Document *doc;
   FILE *f;
   
   while (0 < (ch = getopt(argc, argv, optstring)) ) {
      switch (ch) {
         case 'o': outputfile = strdup(optarg); break;
         default : rtfm();
      }
   }

   if (optind >= argc) rtfm();
   inputfile = strdup(argv[optind]);
   
   if (! (f = fopen(inputfile, "r")) ) {
      fprintf(stderr, "Error opening %s for reading.\n", inputfile);
      exit(1);
   }

   if (! (doc = document_load(f)) ) {
      fprintf(stderr, "Error loading document from %s (bad format?).\n", 
                                                        inputfile);
      exit(1);
   }
   fclose(f);
   
   f = stdout;  /* notice that we are reusing f, which now identifies
                 * the file we are outputting information to */

   if (outputfile && !(f = fopen(outputfile, "w"))) {
      fprintf(stderr, "Error opening %s for writing.\n", outputfile);
      exit(1);
   }

   fprintf(f, "AES v1.0\n");
   fprintf(f, "begin document\n");
   fprintf(f, "layer-count %d\n", doc->layer_count);

   for (i = 0; i < doc->layer_count; i++) {
      Layer *l = doc->layers[i];
      fprintf(f, "begin layer\n");
      fprintf(f, "name %s\n", l->name);
      fprintf(f, "width %d\n", l->width);
      fprintf(f, "height %d\n", l->height);
      fprintf(f, "visible %d\n", l->visible ? 1 : 0);
      fprintf(f, "transparent %d\n", l->visible ? 1 : 0);
      fprintf(f, "begin canvas\n");

      for (y = 0; y < l->height; y++) {
         for (x = 0; x < l->width; x++) {
            fprintf(f, "%x %x ", (int) l->cells[x][y].ch, 
                                 (int) l->cells[x][y].attr);
         }
         fputc('\n', f);
      }
      fprintf(f, "end canvas\n");
      fprintf(f, "end layer\n");
   }

   fprintf(f, "end document\n");
   return 0;
}


