/*
Copyright (c) 2003 Bruno T. C. de Oliveira

LICENSE INFORMATION:
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
Copyright (c) 2002 Bruno T. C. de Oliveira

INFORMAES DE LICENA:
Este programa  um software de livre distribuio; voc pode
redistribu-lo e/ou modific-lo sob os termos da GNU General
Public License, conforme publicado pela Free Software Foundation,
pela verso 2 da licena ou qualquer verso posterior.

Este programa  distribudo na esperana de que ele ser til
aos seus usurios, porm, SEM QUAISQUER GARANTIAS; sem sequer
a garantia implcita de COMERCIABILIDADE ou DE ADEQUAO A
QUALQUER FINALIDADE ESPECFICA. Consulte a GNU General Public
License para obter mais detalhes (uma cpia acompanha este
programa, armazenada no arquivo COPYING).
*/

#include <bores.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>

#include "document.h"

#define MYSYNTAX \
   "Syntax: aecat [-b] [-c] [{-p | -l <layer_num>}] [-o <output_file>] inputfile\n" \
   "\n" \
   "   -c : prepend a 'clear screen' escape sequence\n" \
   "   -o : writes output to specified file rather than stdout\n" \
   "   -b : disables output of color (only characters will be\n" \
   "        printed).\n" \
   "   -h : prints this help text\n" \
   "   -l : specifies which layer of the document is to be used.\n" \
   "        (must be an index, not a layer name). By default,\n" \
   "        layer 0 will be used.\n" \
   "   -p : exports a composite, that is, overlays all visible layers,\n" \
   "        paying attention to layer transparency, etc. The size of the\n" \
   "        composite will be the size of the first layer\n"

void rtfm() {
   fprintf(stderr, MYSYNTAX);
   exit(1);
}

int main(int argc, char **argv) {
   int ch, width, height;
   static char optstring[] = "o:chl:bp";
   ubool clear_screen = ufalse;
   ubool use_color = utrue;
   char *outputfile = NULL;
   char *inputfile;
   Document *doc;
   FILE *f;
   int layer_number = 0; /* -1 means composite */
   int x, y;
   int prev_fg = -1, prev_bg = -1, prev_bold = -1;
   Layer *lyr;
   
   while (0 < (ch = getopt(argc, argv, optstring)) ) {
      switch (ch) {
         case 'c': clear_screen = utrue; break;
         case 'o': outputfile = strdup(optarg); break;
         case 'l': layer_number = atoi(optarg); break;
         case 'p': layer_number = -1; break;
         case 'b': use_color = ufalse; break;
         default : rtfm();
      }
   }

   if (optind >= argc) rtfm();
   inputfile = strdup(argv[optind]);

   if (! (f = fopen(inputfile, "r")) ) {
      fprintf(stderr, "Error opening %s for reading.\n", inputfile);
      exit(1);
   }

   if (! (doc = document_load(f)) ) {
      fprintf(stderr, "Error loading document from %s (bad format?).\n", 
                                                        inputfile);
      exit(1);
   }
   fclose(f);
   
   f = stdout;  /* notice that we are reusing f, which now identifies
                 * the file we are outputting information to */

   if (outputfile && !(f = fopen(outputfile, "w"))) {
      fprintf(stderr, "Error opening %s for writing.\n", outputfile);
      exit(1);
   }

   if (clear_screen) fputs("\e[2J\e[H", f);

   if (layer_number >= doc->layer_count) {
      if (layer_number)
         fprintf(stderr, "Document %s does not have the specified layer: %d\n",
                inputfile, layer_number);
      else
         fprintf(stderr, "Document %s has no layers.\n", inputfile);

      exit(1);
   }

   width = doc->layers[0]->width;
   height = doc->layers[0]->height;

   lyr = layer_number >= 0 ? doc->layers[layer_number] : 0;
   
   for (y = 0; y < height; y++) {
      for (x = 0; x < width; x++) {
         int ch, attr, fg, bg, bold, blink;

         ch = 0x20; attr = 0x70;
         if (lyr) {
            ch = lyr->cells[x][y].ch;
            attr = lyr->cells[x][y].attr;
         }
         else document_calc_effective_cell(doc, x, y, &ch, &attr);

         fg = (attr & 0x70) >> 4;
         bg = (attr & 0x07);
         bold = attr & 0x80;
         blink = attr & 0x08;

         if (use_color && 
                      (fg != prev_fg || bg != prev_bg || bold != prev_bold)) {

            fputs("\e[", f);
            if (bold) fputs("1;", f); else fputs("0;", f);
            if (blink) fputs("5;", f);
            fprintf(f, "%d;%dm", 30 + fg, 40 + bg);

            prev_fg = fg; prev_bg = bg; prev_bold = bold;
         }

         if (ch >= 0 && ch <= 32) ch = ' ';
         fputc(ch, f);
      }

      fputc('\n', f);
   }

   fputs("\e[0m", f);

   return 0;
}


