/*
Copyright (c) 2003 Bruno T. C. de Oliveira

LICENSE INFORMATION:
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
Copyright (c) 2002 Bruno T. C. de Oliveira

INFORMAES DE LICENA:
Este programa  um software de livre distribuio; voc pode
redistribu-lo e/ou modific-lo sob os termos da GNU General
Public License, conforme publicado pela Free Software Foundation,
pela verso 2 da licena ou qualquer verso posterior.

Este programa  distribudo na esperana de que ele ser til
aos seus usurios, porm, SEM QUAISQUER GARANTIAS; sem sequer
a garantia implcita de COMERCIABILIDADE ou DE ADEQUAO A
QUALQUER FINALIDADE ESPECFICA. Consulte a GNU General Public
License para obter mais detalhes (uma cpia acompanha este
programa, armazenada no arquivo COPYING).
*/

#include <bores.h>
#include <stdlib.h>

#include "document.h"

#define DEFAULT_NOM_WIDTH   80
#define DEFAULT_NOM_HEIGHT  25

Document *document_create(void) {
   Document *doc = (Document*) zalloc(sizeof(Document));
   return doc;
}

void document_destroy(Document *doc) {
   int i;
   if (!doc) return;
   if (doc->layers) {
      for (i = 0; i < doc->layer_count; i++)
         free(doc->layers[i]);
      free(doc->layers);
   }

   dstrset(&doc->metainfo, 0);
   free(doc);
}

void document_add_layer(Document *doc, Layer *layer) {
   (doc->layers = srealloc(doc->layers, (++doc->layer_count) * sizeof(Layer*)))
                  [doc->layer_count-1] = layer;
}

void document_insert_layer(Document *doc, int i, Layer *layer) {
   int j;
   if (i < 0) i = 0;
   if (i > doc->layer_count) i = doc->layer_count;

   /* first reallocate the array (and increment doc->layer_count) */
   doc->layers = srealloc(doc->layers, (++doc->layer_count) * sizeof(Layer*));

   /* now shift layers to make room */
   for (j = doc->layer_count - 1; j > i; j--)
      doc->layers[j] = doc->layers[j-1];

   /* position i is now free */
   doc->layers[i] = layer;
}

void document_del_layer(Document *doc, int i) {
   if (i < 0 || i >= doc->layer_count) return;

   /* FIXME: this doesn't realloc the layers array, so it does not
    * reclaim memory allocated by document_add_layer. */
   layer_destroy(doc->layers[i]);
   for (; i < doc->layer_count - 1; i++)
      doc->layers[i] = doc->layers[i+1];

   doc->layer_count--; 
}

void document_get_nom_dim(Document *doc, int *w, int *h) {
   if (!doc || !doc->layer_count) {
      if (w) *w = DEFAULT_NOM_WIDTH;
      if (h) *h = DEFAULT_NOM_HEIGHT;
   }
   else {
      if (w) *w = doc->layers[0]->width;
      if (h) *h = doc->layers[0]->height;
   }
}

void document_save(Document *doc, FILE *f) {
   int i;
   fputc('D', f); fputc('O', f);
   fwrite(&doc->layer_count, sizeof(int), 1, f);

   fsavestr(doc->metainfo ? doc->metainfo : "", f);
   
   for (i = 0; i < doc->layer_count; i++)
      layer_save(doc->layers[i], f);
}

Document *document_load(FILE *f) {
   int i; int lc;
   Document *d; Layer *l;
   if ('D' != fgetc(f) || 'O' != fgetc(f)) return NULL;
   
   fread(&lc, sizeof(int), 1, f);
   
   d = document_create();
   if ( !(d->metainfo = floadstr(f)) ) {
      document_destroy(d);
      return NULL;
   }

   for (i = 0; i < lc; i++) {
      if ( !(l = layer_load(f)) ) {
         document_destroy(d);
         return NULL;
      }
      
      document_add_layer(d, l);
   }

   return d;
}

ubool document_calc_effective_cell(Document *doc, int x, int y, int *ch,
                                   int *attr) {
   int i;
   Layer *lyr;
   int result_ch = 0x20, result_attr = 0x70;

   for (i = doc->layer_count - 1; i >= 0; i--) {
      lyr = doc->layers[i];
      if (x < 0 || x >= lyr->width || y < 0 || y >= lyr->width) continue;
      if (!lyr->visible) continue;
      if (lyr->transp && is_cell_transp(&lyr->cells[x][y])) continue;

      result_ch = lyr->cells[x][y].ch;
      result_attr = lyr->cells[x][y].attr;
   }

   if (ch) *ch = result_ch;
   if (attr) *attr = result_attr;
   return utrue;
}

